@@ -1,6 +1,5 @@
 import dataclasses
 import json
-import operator
 import re
 import string
 from typing import (
@@ -17,6 +16,8 @@ from typing import (
     cast,
 )
 
+from .escape import escape
+
 
 @dataclasses.dataclass(eq=False)
 class Placeholder:
@@ -27,7 +28,7 @@ class Placeholder:
         return f"Placeholder(id={id(self)}, name={repr(self.name)})"
 
 
-@dataclasses.dataclass
+@dataclasses.dataclass(frozen=True)
 class Slot:
     __slots__ = ["name"]
     name: str
@@ -37,8 +38,12 @@ Part = Union[str, Placeholder, Slot, "Fragment"]
 FlatPart = Union[str, Placeholder, Slot]
 
 
+newline_whitespace_re = re.compile(r"\s*\n\s*")
+auto_numbered_re = re.compile(r"[A-Za-z0-9_]")
+
+
 def auto_numbered(field_name):
-    return not re.match(r"[A-Za-z0-9_]", field_name)
+    return not auto_numbered_re.match(field_name)
 
 
 def process_slot_value(
@@ -123,40 +128,68 @@ class Fragment:
                 )
         return Fragment(parts, values)
 
-    def query(self) -> Tuple[str, List[Any]]:
+    def prep_query(self, allow_slots=False):
         parts: List[FlatPart] = []
         values: Dict[Placeholder, Any] = {}
         self.flatten_into(parts, values)
-        placeholder_count = 0
+        args: List[Union[Placeholder, Slot]] = []
         placeholder_ids: Dict[Placeholder, int] = {}
+        slot_ids: Dict[Slot, int] = {}
         out_parts: List[str] = []
         for part in parts:
             if isinstance(part, Slot):
-                raise ValueError(f"Unfilled slot: {repr(part.name)}")
+                if not allow_slots:
+                    raise ValueError(f"Unfilled slot: {repr(part.name)}")
+                if part not in slot_ids:
+                    args.append(part)
+                    slot_ids[part] = len(args)
+                out_parts.append(f"${slot_ids[part]}")
             elif isinstance(part, Placeholder):
                 if part not in placeholder_ids:
-                    placeholder_count += 1
-                    placeholder_ids[part] = placeholder_count
+                    args.append(part)
+                    placeholder_ids[part] = len(args)
                 out_parts.append(f"${placeholder_ids[part]}")
             else:
                 assert isinstance(part, str)
                 out_parts.append(part)
-        placeholder_values = [
-            values[part]
-            for part, i in sorted(placeholder_ids.items(), key=operator.itemgetter(1))
+        return "".join(out_parts).strip(), values, args
+
+    def query(self) -> Tuple[str, List[Any]]:
+        query, values, args = self.prep_query()
+        placeholder_values = [values[arg] for arg in args]
+        return query, placeholder_values
+
+    def prepare(self) -> Tuple[str, Callable[..., List[Any]]]:
+        query, values, args = self.prep_query(allow_slots=True)
+        env = dict()
+        func = [
+            "def generate_args(**kwargs):",
+            " return [",
         ]
-        return "".join(out_parts), placeholder_values
+        for i, arg in enumerate(args):
+            if isinstance(arg, Slot):
+                func.append(f"  kwargs[{repr(arg.name)}],")
+            else:
+                env[f"value_{i}"] = values[arg]
+                func.append(f"  value_{i},")
+        func += [" ]"]
+        exec("\n".join(func), env)
+        return query, env["generate_args"]  # type: ignore
 
     def __iter__(self) -> Iterator[Any]:
         sql, args = self.query()
         return iter((sql, *args))
 
-    def join(self, parts: List["Fragment"]):
+    def join(self, parts: Iterable["Fragment"]) -> "Fragment":
         return Fragment(list(join_parts(parts, infix=self)), {})
 
 
 class SQLFormatter:
-    def __call__(self, fmt: str, *args, **kwargs) -> Fragment:
+    def __call__(
+        self, fmt: str, *args, preserve_formatting=False, **kwargs
+    ) -> Fragment:
+        if not preserve_formatting:
+            fmt = newline_whitespace_re.sub(" ", fmt)
         fmtr = string.Formatter()
         parts: List[Part] = []
         values: Dict[Placeholder, Any] = {}
@@ -187,6 +220,10 @@ class SQLFormatter:
         placeholder = Placeholder("value")
         return Fragment([placeholder], {placeholder: value})
 
+    @staticmethod
+    def escape(value: Any) -> Fragment:
+        return lit(escape(value))
+
     @staticmethod
     def slot(name: str) -> Fragment:
         return Fragment([Slot(name)], {})
@@ -203,21 +240,22 @@ class SQLFormatter:
             return lit(f"{quote_identifier(name)}")
 
     @staticmethod
-    def all(frags: Iterable[Fragment]) -> Fragment:
-        return any_all(list(frags), "AND", "TRUE")
+    def all(parts: Iterable[Fragment]) -> Fragment:
+        return any_all(list(parts), "AND", "TRUE")
 
     @staticmethod
-    def any(frags: Iterable[Fragment]) -> Fragment:
-        return any_all(list(frags), "OR", "FALSE")
+    def any(parts: Iterable[Fragment]) -> Fragment:
+        return any_all(list(parts), "OR", "FALSE")
 
     @staticmethod
-    def list(frags: Iterable[Fragment]) -> Fragment:
-        parts = join_parts(frags, infix=", ")
-        return Fragment(list(parts), {})
+    def list(parts: Iterable[Fragment]) -> Fragment:
+        return Fragment(list(join_parts(parts, infix=", ")), {})
 
     @staticmethod
     def unnest(data: Iterable[Sequence[Any]], types: Iterable[str]) -> Fragment:
-        nested = (nest_for_type(x, t) for x, t in zip(zip(*data), types))
+        nested = list(nest_for_type(x, t) for x, t in zip(zip(*data), types))
+        if not nested:
+            nested = list(nest_for_type([], t) for t in types)
         return Fragment(["UNNEST(", sql.list(nested), ")"], {})
 
 
